@@ -12,7 +12,6 @@ import os
 import shutil
 import logging
 import traceback
-import multiprocessing
 import subprocess
 
 #OpenAccess_EPUB Modules
@@ -23,14 +22,13 @@ from openaccess_epub.utils.images import get_images
 import openaccess_epub.opf as opf
 import openaccess_epub.ncx as ncx
 import openaccess_epub.ops as ops
-from openaccess_epub.article import Article as Article
+from openaccess_epub.article import Article
 
 CACHE_LOCATION = utils.cache_location()
 LOCAL_DIR = os.getcwd()
 
 log = logging.getLogger('Main')
 
-
 def OAEParser():
     """
     This function returns the parser args to the main method.
@@ -51,9 +49,15 @@ def OAEParser():
     parser.add_argument('-c', '--clean', action='store_true', default=False,
                         help='''Use to toggle on cleanup. With this flag,
                                 the pre-zipped output will be removed.''')
-    parser.add_argument('-N', '--no-epubcheck', action='store_false',
+    parser.add_argument('-e', '--no-epubcheck', action='store_false',
                         default=True,
                         help='''Use this to skip ePub validation by EpubCheck.''')
+    parser.add_argument('-d', '--no-dtd-validation', action='store_false',
+                        default=True,
+                        help='''Use this to skip DTD-validation on the input
+                                file(s). This is advised only for use on files
+                                that have already been validated by dtdvalidate
+                                or otherwise.''')
     modes = parser.add_mutually_exclusive_group()
     modes.add_argument('-i', '--input', action='store', default=False,
                        help='''Input may be a path to a local directory, a
@@ -111,17 +115,17 @@ def single_input(args, config=None):
     if 'http:' in args.input:
         raw_name = u_input.url_input(args.input)
         abs_input_path = os.path.join(LOCAL_DIR, raw_name+'.xml')
-        parsed_article = Article(abs_input_path)
+        parsed_article = Article(abs_input_path, validation=args.no_dtd_validation)
     #Fetch by DOI
     elif args.input[:4] == 'doi:':
         raw_name = u_input.doi_input(args.input)
         abs_input_path = os.path.join(LOCAL_DIR, raw_name+'.xml')
-        parsed_article = Article(abs_input_path)
+        parsed_article = Article(abs_input_path, validation=args.no_dtd_validation)
     #Local XML input
     else:
         abs_input_path = utils.get_absolute_path(args.input)
         raw_name = u_input.local_input(abs_input_path)
-        parsed_article = Article(abs_input_path)
+        parsed_article = Article(abs_input_path, validation=args.no_dtd_validation)
 
     #Generate the output path name, this will be the directory name for the
     #output. This output directory will later be zipped into an EPUB
@@ -180,7 +184,8 @@ def batch_input(args, config=None):
         except:
             traceback.print_exc(file=error_file)
         else:
-            parsed_article = Article(os.path.join(args.batch, raw_name+'.xml'))
+            parsed_article = Article(os.path.join(args.batch, raw_name+'.xml'),
+                                     validation=args.no_dtd_validation)
 
         #Create the output name
         output_name = os.path.join(utils.get_output_directory(args), raw_name)
@@ -273,7 +278,7 @@ def collection_input(args, config=None):
     #Now it is time to operate on each of the xml files
     for xml_file in xml_files:
         raw_name = u_input.local_input(xml_file)  # is this used?
-        parsed_article = Article(xml_file)
+        parsed_article = Article(xml_file, validation=args.no_dtd_validation)
         toc.take_article(parsed_article)
         myopf.take_article(parsed_article)
     
@@ -320,90 +325,6 @@ def collection_input(args, config=None):
         epubcheck('{0}.epub'.format(output_name), config)
 
 
-class ParallelBatchProcess(multiprocessing.Process):
-    """
-    
-    """
-    def __init__(self, task_queue, error_file, args):
-        multiprocessing.Process.__init__(self)
-        self.task_queue = task_queue
-        self.error_file = error_file
-        self.args = args
-        #self.result_queue = result_queue
-
-    def run(self):
-        proc_name = self.name
-        while True:
-            next_task = self.task_queue.get()
-            if next_task is None:  # Poison Pill shutdown method
-                print('{0} Exiting'.format(proc_name))
-                self.task_queue.task_done()
-                break
-            #Normal Batch Stuff
-            #Parse the article
-            try:
-                parsed_article, raw_name = u_input.local_input(next_task)
-            except:
-                traceback.print_exc(file=self.error_file)
-            #Create the output name
-            output_name = os.path.join(utils.get_output_directory(self.args), raw_name)
-            try:
-                make_epub(parsed_article,
-                          outdirect=output_name,
-                          explicit_images=None,   # No explicit image path
-                          batch=False,
-                          config=config)
-            except:
-                traceback.print_exc(file=self.error_file)
-            #Cleanup output directory, keeps EPUB and log
-            shutil.rmtree(output_name)
-            #Running epubcheck on the output verifies the validity of the ePub,
-            #requires a local installation of java and epubcheck.
-            #if self.args.no_epubcheck:
-                #epubcheck('{0}.epub'.format(output_name), config)
-            #Report completed task to the JoinableQueue
-            self.task_queue.task_done()
-        return
-
-
-def parallel_batch_input(args, config):
-    """
-    This is a version of the Batch Input Mode that is designed to operate with
-    parallel processes to take advantage of CPUs with multiple cores. It
-    defaults to spawning as many processes as there are cores.
-    """
-    error_file = open('batch_tracebacks.txt', 'w')
-    # Establish communication queue
-    tasks = multiprocessing.JoinableQueue()
-
-    #Start the processes
-    num_processes = multiprocessing.cpu_count() * 2
-    print('Starting {0} processes'.format(num_processes))
-    processes = [ParallelBatchProcess(tasks, error_file, args)
-                 for i in range(num_processes)]
-    for process in processes:
-        process.start()
-
-    #Enqueue the tasks
-    for item in os.listdir(args.parallel_batch):
-        item_path = os.path.join(args.parallel_batch, item)
-        #Skip directories and files without .xml extension
-        _root, extension = os.path.splitext(item)
-        if not os.path.isfile(item_path):
-            continue
-        if not extension == '.xml':
-            continue
-        tasks.put(item)
-
-    # Add a poison pill for each process
-    for i in range(num_processes):
-        tasks.put(None)
-
-    # Wait for all of the tasks to finish
-    tasks.join()
-    error_file.close()
-
-
 def zipped_input(args, config=None):
     """
     Zipped Input Mode is primarily intended as a workflow for Frontiers
@@ -413,11 +334,11 @@ def zipped_input(args, config=None):
         config = get_config_module()
 
 
-def make_epub(document, outdirect, explicit_images, batch, config=None):
+def make_epub(article, outdirect, explicit_images, batch, config=None):
     """
     Encapsulates the primary processing work-flow. Before this method is
     called, pre-processing has occurred to define important directory and file
-    locations. The document has been processed for metadata and now it is time
+    locations. The article has been processed for metadata and now it is time
     to generate the ePub content.
     """
     print('Processing output to {0}.epub'.format(outdirect))
@@ -432,20 +353,17 @@ def make_epub(document, outdirect, explicit_images, batch, config=None):
     epub_base = os.path.join(CACHE_LOCATION, 'base_epub')
     shutil.copytree(epub_base, outdirect)
 
-    if document.metadata.dtdVersion() == '2.0':
-        return
-
     #Get the Digital Object Identifier
-    DOI = document.get_DOI()
+    DOI = article.get_DOI()
 
     #Get the images
-    get_images(DOI, outdirect, explicit_images, config, document)
+    get_images(DOI, outdirect, explicit_images, config, article)
 
     toc = ncx.NCX(__version__, outdirect)
     myopf = opf.OPF(outdirect, False)
-    toc.take_article(document)
-    myopf.take_article(document)
-    ops_doc = ops.OPSPLoS(document, outdirect)
+    toc.take_article(article)
+    myopf.take_article(article)
+    ops_doc = ops.OPSPLoS(article, outdirect)
     toc.write()
     myopf.write()
     utils.epub_zip(outdirect)
