@@ -396,14 +396,18 @@ def epub_zip(outdirect):
     os.chdir(outdirect)
     epub.write('mimetype')
     log.info('Recursively zipping META-INF and OPS')
-    recursive_zip(epub, 'META-INF')
-    recursive_zip(epub, 'OPS')
+    for item in os.listdir('.'):
+        if item == 'mimetype':
+            continue
+        recursive_zip(epub, item)
     os.chdir(current_dir)
     epub.close()
 
 
-def recursive_zip(zipf, directory, folder=''):
+def recursive_zip(zipf, directory, folder=None):
     """Recursively traverses the output directory to construct the zipfile"""
+    if folder is None:
+        folder = ''
     for item in os.listdir(directory):
         if os.path.isfile(os.path.join(directory, item)):
             zipf.write(os.path.join(directory, item), os.path.join(directory,
@@ -465,29 +469,6 @@ def initiateDocument(titlestring,
     return doc, body
 
 
-def plos_fetch_single_representation(article_doi, item_xlink_href):
-    """
-    This function will render a formatted URL for accessing the PLoS' server
-    SingleRepresentation of an object.
-    """
-    #A dict of URLs for PLoS subjournals
-    journal_urls = {'pgen': 'http://www.plosgenetics.org/article/{0}',
-                    'pcbi': 'http://www.ploscompbiol.org/article/{0}',
-                    'ppat': 'http://www.plospathogens.org/article/{0}',
-                    'pntd': 'http://www.plosntds.org/article/{0}',
-                    'pmed': 'http://www.plosmedicine.org/article/{0}',
-                    'pbio': 'http://www.plosbiology.org/article/{0}',
-                    'pone': 'http://www.plosone.org/article/{0}',
-                    'pctr': 'http://clinicaltrials.ploshubs.org/article/{0}'}
-    #Identify subjournal name for base URl
-    subjournal_name = article_doi.split('.')[1]
-    base_url = journal_urls[subjournal_name]
-
-    #Compose the address for fetchSingleRepresentation
-    resource = 'fetchSingleRepresentation.action?uri=' + item_xlink_href
-    return base_url.format(resource)
-
-
 def scrapePLoSIssueCollection(issue_url):
     """
     Uses Beautiful Soup to scrape the PLoS page of an issue. It is used
